local Tail = Core.class(Mesh)

function Tail:init(numPoints, width, color)
	
	self.numPoints = numPoints or 20
	self.width = width or 20
	self.color = color or 0xff0000
	self.linePoints = {}
	self.lineThetas = {}
	self.vertexPoints = {}
	self.indexArray = {}

end

function Tail:addPoint(x, y)
	
	if #self.linePoints >= self.numPoints then
		for i = 3, (#self.linePoints * 2), 1 do
			local x, y = self:getVertex(i)
			self:setVertex(i - 2, x, y)
		end
		table.remove(self.linePoints, 1)
		table.remove(self.lineThetas, 1)
	end
	
	table.insert(self.linePoints, {x = x, y = y})
	
	local curNumPoints = #self.linePoints
	
	if curNumPoints > 1 then
	
		local p1 = self.linePoints[curNumPoints - 1]
		local p2 = self.linePoints[curNumPoints]		
		local lineTheta = math.atan2(p2.y - p1.y, p2.x - p1.x)
		table.insert(self.lineThetas, lineTheta)
		local perpTheta = lineTheta + math.pi/2
		local dx = self.width * math.cos(perpTheta)
		local dy = self.width * math.sin(perpTheta)
		local mx1 = p2.x + dx
		local my1 = p2.y + dy
		local mx2 = p2.x - dx
		local my2 = p2.y - dy
		self:setVertex(curNumPoints * 2 - 1, mx1, my1)
		self:setVertex(curNumPoints * 2, mx2, my2)
		local alpha = (curNumPoints - 1)/(self.numPoints - 1)
		self:setColor(curNumPoints * 2 - 1, self.color, alpha)
		self:setColor(curNumPoints * 2, self.color, alpha)
		local addIndeces = {curNumPoints * 2 - 3, curNumPoints * 2 - 2, curNumPoints * 2 - 1, curNumPoints * 2 - 2, curNumPoints * 2 - 1, curNumPoints * 2}
		for i = 1, #addIndeces, 1 do
			table.insert(self.indexArray, addIndeces[i])
		end
		if curNumPoints == 2 then
			mx1 = p1.x + dx
			my1 = p1.y + dy
			mx2 = p1.x - dx
			my2 = p1.y - dy
			self:setVertex(1, mx1, my1)
			self:setVertex(2, mx2, my2)
			self:setColor(1, self.color, 0)
			self:setColor(2, self.color, 0)
		else
			local prevLineTheta = self.lineThetas[#self.lineThetas - 1]
			local d = self.width/math.sin(perpTheta)
			perpTheta = math.pi * 0.5 + (lineTheta + prevLineTheta)/2
			dx = self.width * math.cos(perpTheta)
			dy = self.width * math.sin(perpTheta)
			mx1 = p1.x + dx
			my1 = p1.y + dy
			mx2 = p1.x - dx
			my2 = p1.y - dy
			self:setVertex((curNumPoints - 1) * 2 - 1, mx1, my1)
			self:setVertex((curNumPoints - 1) * 2, mx2, my2)
		end
		self:setIndexArray(self.indexArray)
			
	end

end

return Tail